"""Automatically generated Default Config.

!!!This files is autogenerated, please do not modify manually.!!!
!!!Change the file default-htw-logger.ini or the default values in the base-classes of htw_logger.config!!!
!!!IF htw_logger.config not exists run ```make install``` before!!!
"""
from cornflakes import config_group


@config_group
class Config:
    """Main config class of the module."""

    # modules


__all__ = ["Config"]
